/*
 *  Box.cpp
 *  Assignment3
 *
 *  Created by Zach Gildersleeve on 2/5/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "Box.h"
#include <iostream.h>

Box::Box(Material *newMaterial, const Point newP1, const Point newP2) : Primitive(newMaterial) {
	p[0] = newP1;
	p[1] = newP2;
	bbox.build(p[0]);
	bbox.build(p[1]);
}

bool Box::intersect(HitRecord &hit, const RenderContext &context, const Ray &ray) const{
	 double tmin, tmax, tymin, tymax, tzmin, tzmax;
	 Vector inverse = ray.inverseDirection;
		 	 
	 tmin = (p[ray.sign.x].x - ray.origin.x) * inverse.x;
	 tmax = (p[1 - ray.sign.x].x - ray.origin.x) * inverse.x;
	 tymin = (p[ray.sign.y].y - ray.origin.y) * inverse.y;
	 tymax = (p[1 - ray.sign.y].y - ray.origin.y) * inverse.y;
	 
	 if ((tmin > tymax) || (tymin > tmax))
		 return false;
	 if (tymin > tmin)
	 {
		 tmin = tymin;
	 }
	 if (tymax < tmax)
	 {
		 tmax = tymax;
	 }
	 
	 tzmin = (p[ray.sign.z].z - ray.origin.z) * inverse.z;
	 tzmax = (p[1 - ray.sign.z].z - ray.origin.z) * inverse.z;
	 
	if ((tmin > tzmax) || (tzmin > tmax))
		return false;
	if (tzmin > tmin)
	{
		tmin = tzmin;
	}
	if (tzmax < tmax)
	{
		tmax = tzmax;
	 }

	if(tmin < tmax)
	{
		if(!hit.hit(tmin, this, material))
		{
			hit.hit(tmax, this, material);
		}
		return true;
	}
	return false;
}


void Box::normal(Vector &normal, const Point &hitPoint, const HitRecord &hit, const RenderContext &context, const Ray &ray) const {

	double  e = 1e-6;
	
	if(fabs(hitPoint.x - this->p[0].x) < e)
		normal = Vector(-1,0,0);
	else if(fabs(hitPoint.x - this->p[1].x) < e)
		normal = Vector(1,0,0);
	else if(fabs(hitPoint.y - this->p[0].y) < e)
		normal = Vector(0,-1,0);
	else if(fabs(hitPoint.y - this->p[1].y) < e)
		normal = Vector(0,1,0);
	else if(fabs(hitPoint.z - this->p[0].z) < e)
		normal = Vector(0,0,-1);
	else
		normal = Vector(0, 0, 1);
}

void Box::getBounds(BoundingBox& bbox) const
{
	bbox.build(p[0]);
	bbox.build(p[1]);
}
