/*
 *  GriddedGroup.cpp
 *  Assignment6
 *
 *  Created by Zach Gildersleeve on 2/26/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "GriddedGroup.h"
#include <iostream>
using namespace std;

GriddedGroup::GriddedGroup(double newNumberOfCells) : numberOfCells(newNumberOfCells) {
	objectCount = 0;
}

void GriddedGroup::addObject(Object *newObject)
{
	bbox.build(newObject->bbox.getMin());
	bbox.build(newObject->bbox.getMax());
	objectCount++;
	objectQueue.push(newObject);
}

void GriddedGroup::getGridCoordinates(const BoundingBox &box, int &sX, int &sY, int &sZ, int &eX, int &eY, int &eZ) const {
	Vector s = (box.getMin() - min) * inverseCellSize - Vector(1.e-6, 1.e-6, 1.e-6);
	sX = Floor(s.x);
	sY = Floor(s.y);
	sZ = Floor(s.z);
	Vector e = (box.getMax() - min) * inverseCellSize + Vector(1.e-6, 1.e-6, 1.e-6);
	eX = Ceil(e.x);
	eY = Ceil(e.y);
	eZ = Ceil(e.z);
}


void GriddedGroup::preprocess()
{
	cout << "\tStarting GriddedGroup preprocess\n";
	
	double t1 = Timer::currentSeconds();
	
	max = bbox.getMax();
	min = bbox.getMin();
	diagonal = max - min;
	diagonal = maxVector(Vector(1.e-4, 1.e-4, 1.e-4), diagonal);
	
	min -= diagonal * 1.e-5;
	max += diagonal * 1.e-5;
	
	diagonal = max - min;
	
	double volume = diagonal.x * diagonal.y * diagonal.z;
	double cbrtVolume = cbrt(volume);
	int targetNumberCells = (int)(objectCount * numberOfCells);
	double averageSide = (double)(cbrt(targetNumberCells));
	cout << "\tInitial Object Count of " << objectCount << " and " << numberOfCells << " Cells\n";
	cout << "\tVolume = " << volume << " targetNumberCells = " << targetNumberCells << "\n";
	cout << "\tBounding Box = " << min.x << " " << min.y << " " << min.z << " : " << max.x << " " << max.y << " " << max.z << "\n";
	
	nx = (int)(averageSide * diagonal.x / cbrtVolume + 0.8);
	nx = nx > 1 ? nx : 1;
	ny = (int)(averageSide * diagonal.y / cbrtVolume + 0.8);
	ny = ny > 1 ? ny : 1;
	nz = (int)(averageSide * diagonal.z / cbrtVolume + 0.8);
	nz = nz > 1 ? nz : 1;
	
	int totalNumberCells = nx * ny * nz;
	
	cout << "\tResizing " << totalNumberCells << " cells in cellList\n";
	
	cellSize = diagonal / Vector(nx, ny, nz);
	inverseCellSize = cellSize.inverse();
	
	cellList.resize(totalNumberCells);
	
	cout << "\tCounting " << objectCount << " Objects...";
	
	Object *tempObject;
	BoundingBox tempObjBbox;
	
	int counter = 0;
	int sX, sY, sZ, eX, eY, eZ;
	int objectCount25 = (int)(objectCount * .25);
	int objectCount50 = (int)(objectCount * .5);
	int objectCount75 = (int)(objectCount * .75);
	
	for (int i = 0; i < objectCount; i++)
	{
		tempObject = objectQueue.front();
		objectQueue.pop();
//		tempObject->getBounds(tempObjBbox);
	
		getGridCoordinates(tempObject->bbox, sX, sY, sZ, eX, eY, eZ);
		for (int x = sX; x < eX; x++)
		{	
			for (int y = sY; y < eY; y++)
			 {
				 for (int z = sZ; z < eZ; z++)
				 {
					 cellList.at(x + (y * nx) + (z * nx * ny)).numberObjects++;
					 counter++;
				 }
			 }
		}
		if (i == objectCount25)
			cout << "\t25%... ";
		if (i == objectCount50)
			cout << "\t50%... ";
		if (i == objectCount75)
			cout << "\t75%... ";
			//cout << "\tProcessing Object " << i << "\n";//Grid Coords = " << sX << " " << sY << " " << sZ << " : " << eX << " " << eY << " " << eZ << " counter of " << counter << "\n";
		objectQueue.push(tempObject);
	}
	cout << " done\n\tCounted " << counter << " Objects\n";
	
	cout << "\tAllocating New Groups...";
	
	int nx25 = (int)(nx * .25);
	int nx50 = (int)(nx * .5);
	int nx75 = (int)(nx * .75);

	counter = 0;
	for (int x = 0; x < nx; x++)
	{
		for (int y = 0; y < ny; y++)
		{
			for (int z = 0; z < nz; z++)
			{
				cellList.at(x + (y * nx) + (z * nx * ny)).objectList = new Group(cellList.at(x + (y * nx) + (z * nx * ny)).numberObjects);
				counter++;
			}
		}
		if (x == nx25)
			cout << "\t25%... ";
		if (x == nx50)
			cout << "\t50%... ";
		if (x == nx75)
			cout << "\t75%... ";
	}
	
	cout << " done\n\tAllocated " << counter << " New Groups\n\tAdding Objects To Cell Groups...";
	counter = 0;
	for (int i = 0; i < objectCount; i++)
	{
		tempObject = objectQueue.front();
		objectQueue.pop();
//		tempObject->getBounds(tempObjBbox);
		
		getGridCoordinates(tempObject->bbox, sX, sY, sZ, eX, eY, eZ);
		for (int x = sX; x < eX; x++)
		{	
			for (int y = sY; y < eY; y++)
			{
				for (int z = sZ; z < eZ; z++)
				{
					cellList.at(x + (y * nx) + (z * nx * ny)).addObject(tempObject);
					counter++;
				}
			}
		}
		if (i == objectCount25)
			cout << "\t25%... ";
		if (i == objectCount50)
			cout << "\t50%... ";
		if (i == objectCount75)
			cout << "\t75%... ";
		//cout << "\tGrid Coords = " << sX << " " << sY << " " << sZ << " : " << eX << " " << eY << " " << eZ << " counter of " << counter << "\n";
		//objectQueue.push(tempObject);
	}
	double t2 = Timer::currentSeconds();
	cout << " done\n\tAdded " << counter << " Objects\n\tGriddedGroup processed in " << t2 - t1 << " seconds\n";
	pt[0] = min;
	pt[1] = max;
}

bool GriddedGroup::intersect(HitRecord &hit, const RenderContext &context, const Ray &ray) const {
	
	double epsilon = 1e-6;
	
	// STEP 2 INTERSECT WITH BOUNDING BOX, RETURN tNear
	double tmin, tmax, tymin, tymax, tzmin, tzmax, tnear, tfar;
	Vector inverse = ray.inverseDirection;
	
	tmin = (pt[ray.sign.x].x - ray.origin.x) * inverse.x;
	tmax = (pt[1 - ray.sign.x].x - ray.origin.x) * inverse.x;
	tymin = (pt[ray.sign.y].y - ray.origin.y) * inverse.y;
	tymax = (pt[1 - ray.sign.y].y - ray.origin.y) * inverse.y;
	
	if ((tmin > tymax) || (tymin > tmax))
		return false;
	if (tymin > tmin)
	{
		tmin = tymin;
	}
	if (tymax < tmax)
	{
		tmax = tymax;
	}
	
	tzmin = (pt[ray.sign.z].z - ray.origin.z) * inverse.z;
	tzmax = (pt[1 - ray.sign.z].z - ray.origin.z) * inverse.z;
	
	if ((tmin > tzmax) || (tzmin > tmax))
		return false;
	if (tzmin > tmin)
	{
		tmin = tzmin;
	}
	if (tzmax < tmax)
	{
		tmax = tzmax;
	}
	
	if (tmin < epsilon && tmax < epsilon)
		return false;
	
	if (tmin < epsilon)
		tnear = 0.0;
	else
		tnear = tmin;
	
	tfar = tmax;
	
	// step 3 COMPUTER LATTICE COORDINATES OF NEAR POINT
	// WORLD SPACE P = O +tNear * V
	// LATTICE SPACE L = (int)((P - Pmin) / cellsize)
	// watch out for roundoff error
	
	Point P = ray.origin + ray.direction * tnear;
	Vector L = P - pt[0];
	
	int Lx = (int)(L.x * inverseCellSize.x);
	if (Lx < 0)
		Lx = 0;
	else if (Lx >= nx)
		Lx = nx - 1;
	
	int Ly = (int)(L.y * inverseCellSize.y);
	if (Ly < 0)
		Ly = 0;
	else if (Ly >= ny)
		Ly = ny - 1;	
	
	int Lz = (int)(L.z * inverseCellSize.z);
	if (Lz < 0)
		Lz = 0;
	else if (Lz >= nz)
		Lz = nz - 1;
		
	// Step 4 determine how ray marching changes index
	// diy = D.y * V.y > 0 ? 1: -1
	// stopy = D.y * V.y > 0 ? yres : -1
	// same for x
	
	double _x = diagonal.x * ray.direction.x;
	double _y = diagonal.y * ray.direction.y;
	double _z = diagonal.z * ray.direction.z;
	
	int dix = (_x > 0) ? 1 : -1;
	int stopx = (_x > 0) ? nx : -1;
	
	int diy = (_y > 0) ? 1 : -1;
	int stopy = (_y > 0) ? ny : -1;
	
	int diz = (_z > 0) ? 1 : -1;
	int stopz = (_z > 0) ? nz : -1;	
	
	// step 5 determine how t value changes with ray marching
	//dtdx = abs(cellsize.x / V.x)
	//dtdy = abs(cellsize.y / V.y)
	
	double dtdx = fabs(cellSize.x * inverse.x);
	double dtdy = fabs(cellSize.y * inverse.y);
	double dtdz = fabs(cellSize.z * inverse.z);
	
	// step 6 determine the far edge of the cell
	// if dix == 1, far.x=(L.x+1)*cellsize.x+Pmin
	// if dix == -1, far.x=L.x*cellsize.x+Pmin
	// same for y
	
	double farX, farY, farZ;
	if (dix == 1)
		farX = (Lx + 1) * cellSize.x + pt[0].x;
	else
		farX = Lx * cellSize.x + pt[0].x;
	
	if (diy == 1)
		farY = (Ly + 1) * cellSize.y + pt[0].y;
	else
		farY = Ly * cellSize.y + pt[0].y;
		
	if (diz == 1)
		farZ = (Lz + 1) * cellSize.z + pt[0].z;
	else
		farZ = Lz * cellSize.z + pt[0].z;
		
	// step 7 determine t values of far slabs
	// tnext_x = (far.x-O.x)/V.x
	// tnext_y = (far.y-O.y)/V.y
	
	double tnext_x = (farX - ray.origin.x) * inverse.x;
	double tnext_y = (farY - ray.origin.y) * inverse.y;
	double tnext_z = (farZ - ray.origin.z) * inverse.z;
	
	while (tnear < tfar)
	{
		if (cellList.at(Lx + (Ly * nx) + (Lz * nx * ny)).numberObjects > 0)
		{
//			cout << "\tFound object in cell " << Lx << " : " << Ly << " : " << Lz << "\n";
			cellList.at(Lx + (Ly * nx) + (Lz * nx * ny)).objectList->intersect(hit, context, ray);
		}
		
		if (tnext_x < tnext_y && tnext_x < tnext_z)
		{
			Lx += dix;
			if (Lx == stopx)
				break;
			tnear = tnext_x;
			tnext_x += dtdx;
		}
		else if (tnext_y < tnext_z)
		{
			Ly += diy;
			if (Ly == stopy)
				break;
			tnear = tnext_y;
			tnext_y += dtdy;		
		}
		else
		{
			Lz += diz;
			if (Lz == stopz)
				break;
			tnear = tnext_z;
			tnext_z += dtdz;
		}
	}
}