/*
 *  Image.cpp
 *  Assignment1
 *
 *  Created by Zach Gildersleeve on 1/17/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "Image.h"
#include <iostream>
#include <fstream>
#include <GLUT/glut.h>


using namespace std;

Image::Image(int w, int h) : width(w), height(h) {
	imageData = new Pixel*[width];
	for (int i = 0; i < width; i++)
	{
		imageData[i] = new Pixel[height];
	}
}

Image::Image(const char *fileName)
{
	int numberColors;
	
	ifstream in(fileName);
	if (!in)
		cout << "Error while opening file: " << fileName << "for reading\n";
	if (in.get() != 'P')
		cout << "Error: wrong file format!\n";
	if (in.get() != '6')
		cout << "Error: wrong file format!\n";
	
	in >> width >> height >> numberColors;
	in.get();
	
	cerr << "Processing " << fileName << "\n \tWidth: " << width << "\n \tHeight: " << height << "\n";

	imageData = new Pixel*[width];
	Pixel *newPixel = new Pixel[height * width];
	
	for (int i = 0; i < height; i++)
	{
		imageData[i] = newPixel + i * width;
	}
	
	Pixel temp;
	
	for (int i = height - 1; i >= 0; i--)
	{
		in.read(reinterpret_cast<char*>(imageData[i]), sizeof(Pixel) * width);
	}
	
	
}

Image::~Image() {
	delete[] imageData[0];
	delete[] imageData;
}

void Image::set(int i, int j, const Color &result) {
	Pixel p;
	if (result.r <= 0.0)
		p.r = 0;
	else if (result.r >= 1.0)
		p.r = 255;
	else
		p.r = (signed char)(result.r * 255);
	
	if (result.g <= 0.0)
		p.g = 0;
	else if (result.g >= 1.0)
		p.g = 255;
	else
		p.g = (signed char)(result.g * 255);
	
	if (result.b <= 0.0)
		p.b = 0;
	else if (result.b >= 1.0)
		p.b = 255;
	else
		p.b = (signed char)(result.b * 255);
	
	imageData[j][i] = p;
//	glColor3f(result.r, result.g, result.b);
//	glVertex2i(i - 256, j - 256);
}

void Image::setPixel(int i, int j, const Color &result) {
		Pixel p;
		if (result.r <= 0.0)
			p.r = 0;
		else if (result.r >= 1.0)
			p.r = 255;
		else
			p.r = (signed char)(result.r * 255);
			
		if (result.g <= 0.0)
			p.g = 0;
		else if (result.g >= 1.0)
			p.g = 255;
		else
			p.g = (signed char)(result.g * 255);
			
		if (result.b <= 0.0)
			p.b = 0;
		else if (result.b >= 1.0)
			p.b = 255;
		else
			p.b = (signed char)(result.b * 255);
		
		imageData[j][i] = p;
		glColor3f(result.r, result.g, result.b);
		glVertex2i(i - (width / 2.0), j - (height / 2.0));
}

void Image::write(const char *name) {
	ofstream myFile;
	
	myFile.open(name);
	
	if (myFile.is_open()) {
		myFile << "P6 " << width << " " << height << " 255\n";
		for (int i = height - 1; i >= 0; i--)
		{
			for (int j = 0; j < width; j++)
			{
				myFile << imageData[i][j].r << imageData[i][j].g << imageData[i][j].b;
			}
		}
		myFile.close();
	}
	else
		cout << "Unable to open file for output";
}

Color Image::interpolate(double x, double y)
{
	x *= width;
	y *= height;
	
	int ix = Floor(x) % width;
	int iy = Floor(y) % height;
	
	if (ix < 0) ix += width;
	if (iy < 0) iy += height;
	
	int ix1 = (ix + 1) % width;
	int iy1 = (iy + 1) % height;
	
	double fx = x - ix;
	double fy = y - iy;
	
	Color c00 = Color(imageData[iy][ix].r, imageData[iy][ix].g, imageData[iy][ix].b);
	Color c01 = Color(imageData[iy][ix1].r, imageData[iy][ix1].g, imageData[iy][ix1].b);
	Color c10 = Color(imageData[iy1][ix].r, imageData[iy1][ix].g, imageData[iy1][ix].b);
	Color c11 = Color(imageData[iy1][ix1].r, imageData[iy1][ix1].g, imageData[iy1][ix1].b);
	
	Color result =	c00 * (1.0 - fx) * (1.0 - fy) +
					c01 * fx * (1.0 - fy) +
					c10 * (1.0 - fx) * fy +
					c11 * fx * fy;
	return result * (1.0 / 255.0); 

}
