/*
 *  Point.h
 *  Assignment1
 *
 *  Created by Zach Gildersleeve on 1/14/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "Vector.h"

#ifndef POINT_H
#define POINT_H

#include <iostream>
using namespace std;

class Vector;

class Point {
	public:
		double x, y, z;		

		Point() {}
		Point(const Point &p) : x(p.x), y(p.y), z(p.z) {}
		Point(double newX, double newY, double newZ) : x(newX), y(newY), z(newZ) {}
		
		~Point() {}
		
		Point &operator=(const Point &p) {
			x = p.x;
			y = p.y;
			z = p.z;
			return *this;
		}
		
		bool operator==(const Point &p) {
			return x == p.x && y == p.y && z == p.z;
		}
		
		bool operator!=(const Point &p) {
			return x != p.x && y != p.y && z != p.z;
		}
		
		// need to be defined outside of Point.h
		inline explicit Point(const Vector &v);
		Point operator+(const Vector &v) const;
		Vector operator-(const Point &p) const;
		Point operator-(const Vector &v) const;
		
		Point &operator+=(const Vector &v) {
			x += v.x;
			y += v.y;
			z += v.z;
			return *this;
		}
		Point &operator-=(const Vector &v) {
			x -= v.x;
			y -= v.y;
			z -= v.z;
			return *this;
		}
};

inline Point::Point(const Vector &v) {
	x = v.x;
	y = v.y;
	x = v.z;
}

inline Point Point::operator+(const Vector &v) const {
	return Point(x + v.x, y + v.y, z + v.z);
}

inline Point operator+(const Vector &v, const Point &p) {
	return Point(v.x + p.x, v.y + p.y, v.z + p.z);
}

inline Vector Point::operator-(const Point &p) const {
	return Vector(x - p.x, y - p.y, z - p.z);
}

inline Point Point::operator-(const Vector &v) const {
	return Point(x - v.x, y - v.y, z - v.z);
}

inline double dot(const Point &a, const Vector &b) {
	return (a.x*b.x + a.y*b.y + a.z*b.z);
}

inline double dot(const Vector &a, const Point &b) {
	return (a.x*b.x + a.y*b.y + a.z*b.z);
}

inline Point minPoint(const Point &p1, const Point &p2)
{
	return Point(min(p1.x, p2.x), min(p1.y, p2.y), min(p1.z, p2.z));
}

inline Point maxPoint(const Point &p1, const Point &p2)
{
	return Point(max(p1.x, p2.x), max(p1.y, p2.y), max(p1.z, p2.z));
}

inline int Floor(double d)
{
	if (d < 0)
	{
		int i = -static_cast<int>(-d);
		if (i == d)
			return i;
		else
			return i-1;
	} else {
		return static_cast<int>(d);
	}
}

inline int Ceil(double d)
{
	if (d < 0)
	{
		int i = -static_cast<int>(-d);
		return i;
	} else {
		int i = static_cast<int>(d);
		if (i == d)
			return i;
		else
			return i+1;
	}
}

#endif