/*
 *  WoodMaterial.cpp
 *  Assignment7
 *
 *  Created by Zach Gildersleeve on 3/16/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "WoodMaterial.h"

WoodMaterial::WoodMaterial()
{
	color1 = Color(240.0/255.0, 208.0/255.0, 120.0/255.0);

//	color1 = Color(82.0/255.0, 55.0/255.0, 13.0/255.0);
	color2 = Color(0.8, 0.8, 0.2);
	
	grainScale = Vector(0.0, 25.5, 25.5);
	grainOctaves = 16;
	grainTurbulence = 6.0;
	grainFrequency = 0.2;
	grainLacunarity = 3.0;
	grainGain = 0.3;
	
	ringScale = Vector(21.5, 41.5, 21.5);
	ringOctaves = 8;
	ringTurbulence = 1.5;					// brightness of white wood
	ringLacunarity = 5.0;					// graininess
	ringGain = 0.8;
	
	Kambient = 0.6;
	Kdiffuse = 0.4;
	specColor = Color(1.0, 1.0, 1.0);
	phongTerm = 120;			
}

void WoodMaterial::shade(Color &result, const RenderContext &context,
								const Ray &ray, const HitRecord &hit,
								int depth, const Color &attenuation) const
{
	// P = O + tV
	Point hitPosition = ray.getOrigin() + (ray.getDirection() * hit.getT());
	
	// Call primitive to get normal - should already be normalized
	Vector normal;
	hit.getPrimitive()->normal(normal, hitPosition, hit, context, ray);
	
	//costheta = N dot V
	double cosTheta = dot(normal, ray.getDirection());
	
	if (cosTheta > 0)
		normal = -normal;
	//	else
	//		cosTheta = -cosTheta;
	
	// color light = scene.ambient * Ka
	Scene *myScene = context.scene;
	Color light = myScene->getAmbient() * Kambient;
	
	const Object *world = myScene->object;
	
	Color currentSpecColor(0, 0, 0);
	
	// loop through all the lights
	for (std::vector<Light*>::iterator iterator = myScene->lightList.begin();
		 iterator < myScene->lightList.end(); iterator++)
	{
		// get C.L (light color) and L (vector to light, normalized in light class)
		Color currentLightColor;
		double distance;
		Vector Lnormal;
		distance =(**iterator).getLight(currentLightColor, Lnormal, context, hitPosition);
		
		// cosphi = N dot Ln
		double cosPhi = dot(normal, Lnormal);
		
		if (cosPhi > 0.0)
		{
			HitRecord shadowHit(distance);
			
			Ray shadowRay(hitPosition, Lnormal);
			world->intersect(shadowHit, context, shadowRay);
			
			if (!shadowHit.getPrimitive()) {
				
				light += currentLightColor * Kdiffuse * cosPhi;
				
				Vector H = Lnormal - ray.getDirection();
				H.normalize();
				
				double cosAlpha = dot(H, normal);
				if (cosAlpha > 0)
				{
					double p = pow(cosAlpha, phongTerm);
					currentSpecColor += currentLightColor * p;
				}
			}
			
		}
	}
	
	Vector textCoord;
	
	hit.getPrimitive()->getTextCoord(textCoord, context, ray, hit);
	
	Point texturePoint = Point(grainScale * textCoord * grainFrequency);
	
	double wood1 = grainTurbulence * turbulenceAbs(grainOctaves, texturePoint, grainLacunarity, grainGain);
	wood1 = wood1 * 0.5 + 0.5;
	
	Color woodColor1 = Color(0.9, 0.9, 0.9) * wood1;
	
	texturePoint = Point(ringScale * textCoord);

	double wood2 = ringTurbulence * turbulenceAbs(ringOctaves, texturePoint, ringLacunarity, ringGain);

	Color woodResult = color1 * (1.0 - wood2) + color2 * wood2;
	
	
	result = light * woodResult * woodColor1 + currentSpecColor * specColor;
	
}