#include "Scene.h"
#include "ConstantBackground.h"
#include "Group.h"
#include "LambertMaterial.h"
#include "PointLight.h"
#include "DirectionalLight.h"
#include "PinholeCamera.h"
#include "OrthographicCamera.h"
#include "Plane.h"
#include "Disc.h"
#include "Ring.h"
#include "Triangle.h"
#include "Box.h"
#include "Cylinder.h"
#include "HeightField.h"
#include "PhongMaterial.h"
#include "MetalMaterial.h"
#include "DielectricMaterial.h"
#include "GriddedGroup.h"
#include "CheckerMaterial.h"
#include "PhongMarbleMaterial.h"
#include "PhongImageMaterial.h"
#include "SpherePolar.h"
#include "WoodMaterial.h"
#include "TileMaterial.h"

static int assignment = 7;

Scene* make_scene()
{
  Scene* scene = new Scene();
  scene->setBackground(new ConstantBackground(Color(1.0, 1.0,1.0)));
  Group* world = new Group();

//  Material* image = new PhongImageMaterial("/Users/zachgildersleeve/Documents/SocS2007/cs6620-RayTracing/Assignment7/land_ocean_ice_cloud_8192.ppm", 0.6, 0.4, Color(1,1,1), 80);
//  world->addObject(new SpherePolar(image,
//                                   Point(-2, -0.5, 2), 1.6,
//                                   Vector(0,-.4,1), Vector(-1,-.2,0)));
//  Material* nightimage = new PhongImageMaterial("/Users/zachgildersleeve/Documents/SocS2007/cs6620-RayTracing/Assignment7/land_lights_16384.ppm", 0.6, 0.4, Color(0,0,0), 80);
//  world->addObject(new SpherePolar(nightimage,
//                                   Point(0, 2.0, 3.5), 1.6,
//                                   Vector(0,-.5,1), Vector(.5,1,0)));

  Material* marbleBlue = new PhongMarbleMaterial(Color(0.1, 0.2, 0.5), // Color 1
                                             Color(0.2, 0.3, 0.8), // Color 2
                                             0.5, // scale
                                             8, // octaves
                                             20.0, // turbulence scale
                                             1.0, // frequency scale
                                             2.0, // lacunarity
                                             0.6, // gain
                                             0.6, 0.4, Color(1,1,1), 120);
											 
  Material * wood = new WoodMaterial();									 
  
  
  world->addObject(new Sphere(wood, Point(0.0, 1.5, 1.5), 1.5));
  
  
//  for (double i = 0; i < 1; i++)
//  {
//  	  double iscale = 4.5 * i;
//	  for (int j = 0; j < 1; j++)
//	  {
//		  double jscale = 4.5 * j;
//		  world->addObject(new Sphere(wood, Point(0.0 + iscale, 0.0 + jscale, 0.5), 0.5));  
//	  }
//  }
  
  


  Material* ground = new CheckerMaterial(Point(0,0,0), Vector(1.0,0.0,0), Vector(0.0,1.0,0),
                                         new MetalMaterial(Color(0.8, 0.8, 0.8), 100),
                                         new PhongMaterial(Color(0.5, 0.5, 0.5), 0.6, 0.4,
                                                           Color(1,1,1), 60));
														   
  Material* tile = new TileMaterial(Point(0,0,0), Vector(3.0,0.0,0), Vector(0.0,3.0,0),
                                         new PhongMaterial(Color(0.9, 0.9, 0.9), 0.6, 0.4, Color(1,1,1), 60),
                                         marbleBlue, 0.1);
  world->addObject(new Triangle(tile,
                                Point(-30,-30,-.1), Point(30,-30,-.1), Point(30,30,-.1)));
  world->addObject(new Triangle(tile,
                                Point(-30,-30,-.1), Point(30,30,-.1), Point(-30,30,-.1)));

  scene->setObject(world);

  scene->setAmbient(Color(.4, .4, .4));
  scene->addLight(new PointLight(Point(100, -80, 100), Color(1,1,1)));

  scene->setCamera(new PinholeCamera(Point(1, -15, 4.0),
                                     Point(-0.5,.75, 2),
                                     Vector(0, 0, 1),
                                     25));
  scene->setMaxRayDepth(10);
  scene->setMinAttenuation(.01);

  return scene;
}
